#ifndef TRANSPORT_H
#define TRANSPORT_H
#include "lrt.h"
class Integrator {
public:
	virtual ~Integrator();
	virtual Spectrum L(const Scene *scene,
		const Ray &ray,	Float *alpha) const = 0;
	virtual Spectrum Shade(const Scene *scene,
			       const Ray &ray, Float *alpha,
			       Surf& hit)
	  {Ray r(ray.O, ray.D);
	  return L(scene, r, alpha);}
};
class RayCastingIntegrator : public Integrator {
public:
	Spectrum L(const Scene *scene, const Ray &ray,
		Float *alpha) const;
};
class WhittedIntegrator : public Integrator {
public:
	WhittedIntegrator(int md) {
		MaxDepth = md;
	}
	Spectrum L(const Scene *scene, const Ray &ray, Float *alpha = NULL) const;
	virtual Spectrum Shade(const Scene *scene,
			       const Ray &ray, Float *alpha,
			       Surf& hit);
	static int RayDepth;
private:
	int MaxDepth;
};
class PathIntegrator : public Integrator {
public:
	Spectrum L(const Scene *scene, const Ray &ray, Float *alpha) const;
};
class MCIntegrator : public Integrator {
public:
	Spectrum L(const Scene *scene, const Ray &ray, Float *alpha) const;
};
#endif // TRANSPORT_H
